@@ -5,26 +5,71 @@ setMethodS3("bootstrapCIs", "PairedPSCBS", function(fit, ...) {
 }, private=TRUE)
 
 
-setMethodS3("callSegments", "PairedPSCBS", function(fit, ...) {
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # 2. Calling regions
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # Call region
-  # a. Classifying non-LOH regions as balanced or not balanced
-  # b. Testing for LOH in the Tumor
-}) # callSegments()
-
-
-
+###########################################################################/**
+# @set "class=PairedPSCBS"
+# @RdocMethod extractTCNAndDHs
+#
+# @title "Extract TCN and DH mean levels per segment"
+#
+# \description{
+#   @get "title".
+# }
+# 
+# @synopsis
+#
+# \arguments{
+#  \item{...}{Arguments passed to \code{getSegments()}.}
+# }
+#
+# \value{
+#   Returns a @data.frame.
+# }
+#
+# @author
+#
+# \seealso{
+#   @seemethod "extractMinorMajorCNs".
+#   @seeclass
+# }
+#*/###########################################################################  
 setMethodS3("extractTCNAndDHs", "PairedPSCBS", function(fit, ...) {
-  segs <- fit$output;
+  segs <- getSegments(fit, ...);
   stopifnot(!is.null(segs));
 
   data <- segs[,c("tcnMean", "dhMean", "tcnNbrOfLoci", "dhNbrOfLoci"), drop=FALSE];
   data;
-})
+}, protected=TRUE)
 
 
+
+###########################################################################/**
+# @set "class=PairedPSCBS"
+# @RdocMethod extractMinorMajorCNs
+# @aliasmethod extractC1C2
+#
+# @title "Extract minor and major copy-number mean levels per segment"
+#
+# \description{
+#   @get "title".
+# }
+# 
+# @synopsis
+#
+# \arguments{
+#  \item{...}{Not used.}
+# }
+#
+# \value{
+#   Returns a @data.frame.
+# }
+#
+# @author
+#
+# \seealso{
+#   @seemethod "extractTCNAndDHs"
+#   @seeclass
+# }
+#*/###########################################################################  
 setMethodS3("extractMinorMajorCNs", "PairedPSCBS", function(fit, ...) {
   data <- extractTCNAndDHs(fit, ...);
 
@@ -38,19 +83,26 @@ setMethodS3("extractMinorMajorCNs", "PairedPSCBS", function(fit, ...) {
   colnames(data)[1:2] <- c("C1", "C2");
 
   data;
-}, private=TRUE)
+}, protected=TRUE)
 
 
 setMethodS3("extractC1C2", "PairedPSCBS", function(...) {
   extractMinorMajorCNs(...);
+}, protected=TRUE)
+
+
+setMethodS3("extractCNs", "PairedPSCBS", function(fit, splitters=TRUE, ...) {
+  data <- extractC1C2(fit, splitters=splitters, ...);
+  data[,c("C1", "C2"), drop=FALSE];
 })
 
+
 setMethodS3("extractDeltaC1C2", "PairedPSCBS", function(...) {
   xy <- extractC1C2(...);
   X <- xy[,1:2,drop=FALSE];
   dX <- matrixStats::colDiffs(X);
   dX;
-}, private=TRUE)
+}, protected=TRUE)
 
 
 
@@ -83,9 +135,9 @@ setMethodS3("postsegmentTCN", "PairedPSCBS", function(fit, ..., force=FALSE, ver
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   # Extract the data and segmentation results
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  data <- fit$data;
+  data <- getLocusData(fit);
 
-  segs <- fit$output;
+  segs <- getSegments(fit);
   keep <- is.finite(segs$chromosome);
   segs <- segs[keep,,drop=FALSE];
   tcnSegRows <- fit$tcnSegRows[keep,,drop=FALSE];
@@ -319,151 +371,14 @@ setMethodS3("postsegmentTCN", "PairedPSCBS", function(fit, ..., force=FALSE, ver
 
 
 
-setMethodS3("extractByRegion", "PairedPSCBS", function(this, region, ...) {
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  # Validate arguments
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # Argument 'region':
-  region <- Arguments$getIndex(region, max=nbrOfSegments(this));
-
-  extractByRegions(this, regions=region, ...);
-}, private=TRUE) # extractByRegion()
-
-
-
-setMethodS3("extractByRegions", "PairedPSCBS", function(this, regions, ..., verbose=FALSE) {
-  fit <- this;
-
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  # Local functions
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  updateSegRows <- function(segRows, regions) {
-    segRows <- segRows[regions,,drop=FALSE];
-    ns <- segRows[,2] - segRows[,1] + 1L;
-    from <- c(1L, cumsum(ns)[-length(ns)]);
-    to <- from + (ns - 1L);
-    segRows[,1] <- from;
-    segRows[,2] <- to;
-    verbose && str(verbose, segRows);
-    segRows;
-  } # updateSegRows()
-
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  # Validate arguments
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # Argument 'regions':
-  regions <- Arguments$getIndices(regions, max=nbrOfSegments(fit));
-
-  # Argument 'verbose':
-  verbose <- Arguments$getVerbose(verbose);
-  if (verbose) {
-    pushState(verbose);
-    on.exit(popState(verbose));
-  }
-
- 
-  verbose && enter(verbose, "Extracting subset by regions");
-
-  verbose && cat(verbose, "Number of segments: ", length(regions));
-  verbose && str(verbose, regions);
-
-
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  # Extract data and estimates
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  data <- fit$data;
-  tcnSegRows <- fit$tcnSegRows;
-  dhSegRows <- fit$dhSegRows;
-  segs <- fit$output;
-  params <- fit$params;
-
-  # Sanity checks
-  stopifnot(all(!is.na(data$chromosome) & !is.na(data$x)));
-  stopifnot(length(tcnSegRows) == length(dhSegRows));
-
-  # Sanity checks
-  if (!params$joinSegments) {
-    throw("Cannot bootstrap TCN and DH by segments unless PSCNs are segmented using joinSegments=TRUE.");
-  } 
-
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # Subset segments
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  verbose && enter(verbose, "Update 'output'");
-  segsT <- segs[regions,,drop=FALSE];
-  verbose && str(verbose, segsT);
-  verbose && exit(verbose);
-
-
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # Subset data accordingly
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  verbose && enter(verbose, "Update 'data'");
-
-  segRows <- tcnSegRows;
-  segRows <- segRows[regions,,drop=FALSE];
-  from <- segRows[[1]];
-  to <- segRows[[2]];
-  ok <- (!is.na(from) & !is.na(to));
-  from <- from[ok];
-  to <- to[ok];
-  keep <- logical(nrow(data));
-  for (rr in seq(along=from)) {
-    keep[from[rr]:to[rr]] <- TRUE;
-  }
-  keep <- which(keep);
-  verbose && printf(verbose, "Identified %d (%.2f%%) of %d data rows:\n", length(keep), 100*length(keep)/nrow(data), nrow(data));
-  verbose && str(verbose, keep);
-
-  dataT <- data[keep,,drop=FALSE];
-  verbose && str(verbose, dataT);
-
-  verbose && exit(verbose);
-
-
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  # Update 'segRows'
-  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  verbose && enter(verbose, "Update 'segRows'");
-
-  segRows <- updateSegRows(tcnSegRows, regions=regions);
-  d <- tcnSegRows[regions,] - segRows;
-  # Sanity check
-  stopifnot(identical(d[,1], d[,2]));
-  d <- d[,1];
-  verbose && cat(verbose, "Row deltas:");
-  verbose && str(verbose, d);
-
-  tcnSegRows <- tcnSegRows[regions,,drop=FALSE] - d;
-  verbose && str(verbose, tcnSegRows);
-  # Sanity checks
-  stopifnot(max(tcnSegRows, na.rm=TRUE) <= nrow(dataT));
-
-  dhSegRows <- dhSegRows[regions,,drop=FALSE] - d;
-  verbose && str(verbose, dhSegRows);
-  # Sanity checks
-  stopifnot(max(dhSegRows, na.rm=TRUE) <= nrow(dataT));
-
-  verbose && exit(verbose);
-
-
-  # Create new object
-  res <- fit;
-  res$data <- dataT;
-  res$output <- segsT;
-  res$tcnSegRows <- tcnSegRows;
-  res$dhSegRows <- dhSegRows;
-
-  verbose && exit(verbose);
-
-  res;
-}, private=TRUE) # extractByRegions()
-
-
-
-
 ############################################################################
 # HISTORY:
+# 2011-10-16
+# o Added extractCNs().
+# 2011-10-14
+# o Now extractTCNAndDHs() passes '...' to getSegments().
+# 2011-10-02
+# o CLEANUP: Dropped empty callSegments() for PairedPSCBS.
 # 2011-06-14
 # o Updated code to recognize new column names.
 # 2011-04-08
